# 性能测试

使用[autocannon](https://www.npmjs.com/package/autocannon)进行压测，与[express.js](https://www.expressjs.com.cn/)进行对比。

## 性能对比

![express-vs-qz](imgs/express-vs-qz.png)

## 环境

- 服务器: 阿里云ECS（2核4G，CPU已使用0%，内存已使用20%）
- nodejs: 18.18.2
- npm: 10.2.4

## autocannon

### 安装

```shell
npm i autocannon -g
```

### 使用

- -c，代表并发连接数，经测试10-100效果相似，这里使用10
- -d，持续时间，较长的持续时间可以屏蔽测试机本身性能的波动，这里使用10s

```shell
autocannon -p 1 -c 10 -d 10 http://localhost:9999
```

## 启动代码

[https://github.com/uikoo9/qiao-z-autocannon](https://github.com/uikoo9/qiao-z-autocannon)

### express.js

```js
const express = require('express');
const app = express();

// render
app.engine('html', require('express-art-template'));
app.set('views', __dirname);
app.set('view engine', 'html');

// controller
require('./TestController.js')(app);

// listen
app.listen(9998);
```

### qiao-z

```js
// qz
const qz = require('qiao-z');

// init
(async function () {
  // app
  const app = await qz();

  // listen
  app.listen(9999);
})();
```

## res.send

### 测试代码

```js
app.get('/res-send', function (req, res) {
  res.send('test res.send');
});
```

### 测试结果（QPS）

#### express.js：4,887

![express-res-send](imgs/express-res-send.png)

#### qiao-z：17,858.55

![qz-res-send](imgs/qz-res-send.png)

## res.json

### 测试代码

```js
app.get('/res-json', function (req, res) {
  res.json({ name: 'test res.json' });
});
```

### 测试结果（QPS）

#### express.js：4,889.9

![express-res-json](imgs/express-res-json.png)

#### qiao-z：14,292.6

![qz-res-json](imgs/qz-res-json.png)

## res.render

### 测试代码

```js
app.get('/res-render', function (req, res) {
  res.render('./res-render.html');
});
```

```html
<!doctype html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <title>test res render</title>
  </head>
  <body>
    test res.render
  </body>
</html>
```

### 测试结果（QPS）

#### express.js：2,689.6

![express-res-render](imgs/express-res-render.png)

#### qiao-z：8,197.21

![qz-res-render](imgs/qz-res-render.png)

## res.render(template)

### 测试代码

```js
app.get('/res-render-template', function (req, res) {
  res.render('./res-render-template.html', { name: 'test res.render with template' });
});
```

```html
<!doctype html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <title>test res render</title>
  </head>
  <body>
    {{name}}
  </body>
</html>
```

### 测试结果（QPS）

#### express.js：2,544.81

![express-res-render](imgs/express-res-render-template.png)

#### qiao-z：5,095.3

![qz-res-render](imgs/qz-res-render-template.png)
